//*********************************************************
// www.Ultimategameprogramming.com
// Per-Pixel Lighting shader demo.
// Created by the Programming Ace.
//
// Input Variables used:
//    modelViewProj  - Holds the model view projection matrix.
//    modelView      - Holds model view matrix.
//    lightColor     - Holds light color.
//    lightPos       - Holds light position.
//    ambientMat     - Ambient material.
//    diffuseMat     - Diffuse material.
//    ambientCol     - Ambient color.
//    decal          - Holds the texture.
//    cameraPos      - Position of the view.
//    specularMat    - Specular ligthing material.
//    shininess      - How shinny the objects look.
//    emissive       - Emissive amount of the object's material.
//
//*********************************************************


// Shader variables.  The first is binded to the WorldViewProjection register.
float4x4 modelViewProj : WorldViewProjection;
float4x4 modelView : WorldView;
float4 lightColor;
float4 lightPos;
float4 ambientMat;
float4 ambientCol;
float4 diffuseMat;
float3 cameraPos;
float3 emissive;
float shininess;
float4 specularMat;

texture decal;

sampler Sampler = sampler_state
{
    Texture   = (decal);
};


// Vertex input.
struct App_Input
{
   float3 vertexPos  : POSITION;
   float3 normal     : NORMAL;
   float2 texture0   : TEXCOORD0;
};

struct Sphere_Input
{
   float3 vertexPos  : POSITION;
};

// Vertex output.
struct Vs_Output
{
   float4 vertexPos  : POSITION;
   float2 texture0   : TEXCOORD0;
   float4 position   : TEXCOORD1;
   float3 normal     : TEXCOORD2;
};

struct Sphere_Output
{
   float4 vertexPos  : POSITION;
   float4 color      : COLOR0;
};

// Pixel output.
struct Ps_Output
{
	float4 color      : COLOR;
};


// The vertex shader...
Vs_Output PerPixelLightVS(App_Input IN)
{
   Vs_Output vs_out;

   vs_out.vertexPos = mul(modelViewProj, float4(IN.vertexPos, 1));
   vs_out.texture0 = IN.texture0;

   vs_out.position = float4(IN.vertexPos, 1);
   vs_out.normal = IN.normal;   

   return vs_out;
}


// The pixel shader...
Ps_Output PerPixelLightPS(Vs_Output IN)
{
   Ps_Output ps_out;
   
   // Calculate the ambient color.
   float3 ambient = ambientMat * ambientCol;
   float3 normal = normalize(IN.normal);

   // Calculate the diffuse color.
   float3 lightVec = normalize(lightPos - IN.position);
   float dp = max(dot(normal, lightVec), 0);
   float3 diffuse = diffuseMat * lightColor * dp;
   
   // Calculate the specular highlights.  Get the eye vector, then half angle,
   // then calculate the specular amount by doing pow(n dot half, to the
   // shininess exponet) (normal dot half angle to the shininess power).
   float3 eyeVec = normalize(cameraPos - IN.position);
   float3 halfVec = normalize(lightVec + eyeVec);
   float specularLight = pow(max(dot(normal, halfVec), 0), shininess);

   // If there is no diffuse light, there can't be any specular.
   if(dp <= 0) specularLight = 0;
   
   // Calculate specular amount.
   float3 specular = specularMat * lightColor * specularLight;

   // Calculate the final color as ambient + diffuse.
   ps_out.color.xyz = (emissive + ambient + diffuse + specular) *
                      tex2D(Sampler, IN.texture0);
   ps_out.color.w = 1;

   return ps_out;
}


Sphere_Output LightEffectVS(Sphere_Input IN)
{
   Sphere_Output vs_out;

   vs_out.vertexPos = mul(modelViewProj, float4(IN.vertexPos, 1));
   vs_out.color = lightColor;
   
   return vs_out;
}

Ps_Output LightEffectPS(Sphere_Output IN)
{
   Ps_Output ps_out;

   ps_out.color = IN.color;
   return ps_out;
}


technique PerPixelLighting
{
   pass Pass0
   {
      Sampler[0] = (Sampler);

      // Here we will compile and set the vertex and pixel shader.
      VertexShader = compile vs_2_0 PerPixelLightVS();
      PixelShader  = compile ps_2_0 PerPixelLightPS();
   }
}

technique SimpleLight
{
   pass Pass0
   {
      // Here we will compile and set the vertex and pixel shader.
      VertexShader = compile vs_2_0 LightEffectVS();
      PixelShader  = compile ps_2_0 LightEffectPS();
   }
}